@@ -54,7 +54,14 @@ from .ctx import (
     WebsocketContext,
 )
 from .globals import g, request, session, websocket
-from .helpers import find_package, get_debug_flag, get_env, get_flashed_messages, url_for
+from .helpers import (
+    _split_blueprint_path,
+    find_package,
+    get_debug_flag,
+    get_env,
+    get_flashed_messages,
+    url_for,
+)
 from .json import JSONDecoder, JSONEncoder, jsonify, tojson_filter
 from .logging import create_logger, create_serving_logger
 from .routing import QuartMap, QuartRule
@@ -95,6 +102,9 @@ from .typing import (
     ResponseReturnValue,
     StatusCode,
     TeardownCallable,
+    TemplateFilterCallable,
+    TemplateGlobalCallable,
+    TemplateTestCallable,
     TestAppProtocol,
     TestClientProtocol,
 )
@@ -430,7 +440,7 @@ class Quart(Scaffold):
         """
         processors = self.template_context_processors[None]
         if has_request_context():
-            for blueprint in self._request_blueprints():
+            for blueprint in _request_ctx_stack.top.request.blueprints:
                 if blueprint in self.template_context_processors:
                     processors = chain(  # type: ignore
                         processors, self.template_context_processors[blueprint]
@@ -601,7 +611,9 @@ class Quart(Scaffold):
 
             self.view_functions[endpoint] = view_func
 
-    def template_filter(self, name: Optional[str] = None) -> Callable:
+    def template_filter(
+        self, name: Optional[str] = None
+    ) -> Callable[[TemplateFilterCallable], TemplateFilterCallable]:
         """Add a template filter.
 
         This is designed to be used as a decorator. An example usage,
@@ -616,13 +628,13 @@ class Quart(Scaffold):
             name: The filter name (defaults to function name).
         """
 
-        def decorator(func: Callable) -> Callable:
+        def decorator(func: TemplateFilterCallable) -> TemplateFilterCallable:
             self.add_template_filter(func, name=name)
             return func
 
         return decorator
 
-    def add_template_filter(self, func: Callable, name: Optional[str] = None) -> None:
+    def add_template_filter(self, func: TemplateFilterCallable, name: Optional[str] = None) -> None:
         """Add a template filter.
 
         This is designed to be used on the application directly. An
@@ -641,7 +653,9 @@ class Quart(Scaffold):
         """
         self.jinja_env.filters[name or func.__name__] = func
 
-    def template_test(self, name: Optional[str] = None) -> Callable:
+    def template_test(
+        self, name: Optional[str] = None
+    ) -> Callable[[TemplateTestCallable], TemplateTestCallable]:
         """Add a template test.
 
         This is designed to be used as a decorator. An example usage,
@@ -656,13 +670,13 @@ class Quart(Scaffold):
             name: The test name (defaults to function name).
         """
 
-        def decorator(func: Callable) -> Callable:
+        def decorator(func: TemplateTestCallable) -> TemplateTestCallable:
             self.add_template_test(func, name=name)
             return func
 
         return decorator
 
-    def add_template_test(self, func: Callable, name: Optional[str] = None) -> None:
+    def add_template_test(self, func: TemplateTestCallable, name: Optional[str] = None) -> None:
         """Add a template test.
 
         This is designed to be used on the application directly. An
@@ -681,7 +695,9 @@ class Quart(Scaffold):
         """
         self.jinja_env.tests[name or func.__name__] = func
 
-    def template_global(self, name: Optional[str] = None) -> Callable:
+    def template_global(
+        self, name: Optional[str] = None
+    ) -> Callable[[TemplateGlobalCallable], TemplateGlobalCallable]:
         """Add a template global.
 
         This is designed to be used as a decorator. An example usage,
@@ -696,13 +712,13 @@ class Quart(Scaffold):
             name: The global name (defaults to function name).
         """
 
-        def decorator(func: Callable) -> Callable:
+        def decorator(func: TemplateGlobalCallable) -> TemplateGlobalCallable:
             self.add_template_global(func, name=name)
             return func
 
         return decorator
 
-    def add_template_global(self, func: Callable, name: Optional[str] = None) -> None:
+    def add_template_global(self, func: TemplateGlobalCallable, name: Optional[str] = None) -> None:
         """Add a template global.
 
         This is designed to be used on the application directly. An
@@ -861,8 +877,11 @@ class Quart(Scaffold):
         :func:`~quart.helpers.url_for`.
         """
         functions = self.url_default_functions[None]
-        for blueprint in self._request_blueprints():
-            functions = chain(functions, self.url_default_functions[blueprint])  # type: ignore
+        if "." in endpoint:
+            blueprints = _split_blueprint_path(endpoint.rsplit(".", 1)[0])
+
+            for blueprint in blueprints:
+                functions = chain(functions, self.url_default_functions[blueprint])  # type: ignore
 
         for function in functions:
             function(endpoint, values)
@@ -882,8 +901,14 @@ class Quart(Scaffold):
     def _find_error_handler(self, error: Exception) -> Optional[ErrorHandlerCallable]:
         error_type, error_code = self._get_error_type_and_code(type(error))
 
+        blueprints = []
+        if _request_ctx_stack.top is not None:
+            blueprints = _request_ctx_stack.top.request.blueprints
+        elif _websocket_ctx_stack.top is not None:
+            blueprints = _websocket_ctx_stack.top.websocket.blueprints
+
         for code in [error_code, None]:
-            for name in chain(self._request_blueprints(), [None]):
+            for name in chain(blueprints, [None]):
                 handlers = self.error_handler_spec[name].get(code)
 
                 if handlers is None:
@@ -1074,7 +1099,7 @@ class Quart(Scaffold):
                 omits this argument.
         """
         functions = self.teardown_request_funcs[None]
-        for blueprint in self._request_blueprints():
+        for blueprint in _request_ctx_stack.top.request.blueprints:
             functions = chain(functions, self.teardown_request_funcs[blueprint])  # type: ignore
 
         for function in functions:
@@ -1093,7 +1118,7 @@ class Quart(Scaffold):
                 omits this argument.
         """
         functions = self.teardown_websocket_funcs[None]
-        for blueprint in self._request_blueprints():
+        for blueprint in _websocket_ctx_stack.top.websocket.blueprints:
             functions = chain(functions, self.teardown_websocket_funcs[blueprint])  # type: ignore
 
         for function in functions:
@@ -1477,13 +1502,13 @@ class Quart(Scaffold):
                 omits this argument.
         """
         processors = self.url_value_preprocessors[None]
-        for blueprint in self._request_blueprints():
+        for blueprint in (request_context or _request_ctx_stack.top).request.blueprints:
             processors = chain(processors, self.url_value_preprocessors[blueprint])  # type: ignore
         for processor in processors:
             processor(request.endpoint, request.view_args)
 
         functions = self.before_request_funcs[None]
-        for blueprint in self._request_blueprints():
+        for blueprint in _request_ctx_stack.top.request.blueprints:
             functions = chain(functions, self.before_request_funcs[blueprint])  # type: ignore
         for function in functions:
             result = await self.ensure_async(function)()
@@ -1546,7 +1571,7 @@ class Quart(Scaffold):
                 omits this argument.
         """
         functions = (request_context or _request_ctx_stack.top)._after_request_functions
-        for blueprint in self._request_blueprints():
+        for blueprint in (request_context or _request_ctx_stack.top).request.blueprints:
             functions = chain(functions, self.after_request_funcs[blueprint])
         functions = chain(functions, self.after_request_funcs[None])
 
@@ -1598,13 +1623,13 @@ class Quart(Scaffold):
                 omits this argument.
         """
         processors = self.url_value_preprocessors[None]
-        for blueprint in self._request_blueprints():
+        for blueprint in _websocket_ctx_stack.top.websocket.blueprints:
             processors = chain(processors, self.url_value_preprocessors[blueprint])  # type: ignore
         for processor in processors:
             processor(websocket.endpoint, websocket.view_args)
 
         functions = self.before_websocket_funcs[None]
-        for blueprint in self._request_blueprints():
+        for blueprint in _websocket_ctx_stack.top.websocket.blueprints:
             functions = chain(functions, self.before_websocket_funcs[blueprint])  # type: ignore
         for function in functions:
             result = await self.ensure_async(function)()
@@ -1667,7 +1692,7 @@ class Quart(Scaffold):
                 omits this argument.
         """
         functions = (websocket_context or _websocket_ctx_stack.top)._after_websocket_functions
-        for blueprint in self._request_blueprints():
+        for blueprint in _websocket_ctx_stack.top.websocket.blueprints:
             functions = chain(functions, self.after_websocket_funcs[blueprint])
         functions = chain(functions, self.after_websocket_funcs[None])
 
@@ -1743,19 +1768,6 @@ class Quart(Scaffold):
                 else:
                     raise RuntimeError("While serving generator didn't terminate")
 
-    def _request_blueprints(self) -> Iterable[str]:
-        if _request_ctx_stack.top is not None:
-            blueprint = _request_ctx_stack.top.request.blueprint
-        elif _websocket_ctx_stack.top is not None:
-            blueprint = _websocket_ctx_stack.top.websocket.blueprint
-        else:
-            blueprint = None
-
-        if blueprint is None:
-            return []
-        else:
-            return reversed(blueprint.split("."))
-
 
 def _cancel_all_tasks(loop: asyncio.AbstractEventLoop) -> None:
     tasks = [task for task in asyncio.all_tasks(loop) if not task.done()]
