@@ -59,6 +59,10 @@ class Blueprint(Scaffold):
         cli_group: Optional[str] = Ellipsis,  # type: ignore
     ) -> None:
         super().__init__(import_name, static_folder, static_url_path, template_folder, root_path)
+
+        if "." in name:
+            raise ValueError("Blueprint names may not contain dot '.' characters.")
+
         self.name = name
         self.url_prefix = url_prefix
         self.deferred_functions: List[DeferredSetupFunction] = []
@@ -77,14 +81,15 @@ class Blueprint(Scaffold):
         rule: str,
         endpoint: Optional[str] = None,
         view_func: Optional[Callable] = None,
+        provide_automatic_options: Optional[bool] = None,
         *,
         methods: Optional[Iterable[str]] = None,
         defaults: Optional[dict] = None,
         host: Optional[str] = None,
         subdomain: Optional[str] = None,
-        provide_automatic_options: Optional[bool] = None,
         is_websocket: bool = False,
         strict_slashes: Optional[bool] = None,
+        merge_slashes: Optional[bool] = None,
     ) -> None:
         """Add a route/url rule to the blueprint.
 
@@ -108,17 +113,20 @@ class Blueprint(Scaffold):
                 rule,
                 endpoint,
                 view_func,
+                provide_automatic_options=provide_automatic_options,
                 methods=methods,
                 defaults=defaults,
                 host=host,
                 subdomain=subdomain or self.subdomain,
-                provide_automatic_options=provide_automatic_options,
                 is_websocket=is_websocket,
                 strict_slashes=strict_slashes,
+                merge_slashes=merge_slashes,
             )
         )
 
-    def app_template_filter(self, name: Optional[str] = None) -> Callable:
+    def app_template_filter(
+        self, name: Optional[str] = None
+    ) -> Callable[[TemplateFilterCallable], TemplateFilterCallable]:
         """Add an application wide template filter.
 
         This is designed to be used as a decorator, and has the same arguments
@@ -157,7 +165,9 @@ class Blueprint(Scaffold):
         """
         self.record_once(lambda state: state.register_template_filter(func, name))
 
-    def app_template_test(self, name: Optional[str] = None) -> Callable:
+    def app_template_test(
+        self, name: Optional[str] = None
+    ) -> Callable[[TemplateTestCallable], TemplateTestCallable]:
         """Add an application wide template test.
 
         This is designed to be used as a decorator, and has the same arguments
@@ -194,7 +204,9 @@ class Blueprint(Scaffold):
         """
         self.record_once(lambda state: state.register_template_test(func, name))
 
-    def app_template_global(self, name: Optional[str] = None) -> Callable:
+    def app_template_global(
+        self, name: Optional[str] = None
+    ) -> Callable[[TemplateGlobalCallable], TemplateGlobalCallable]:
         """Add an application wide template global.
 
         This is designed to be used as a decorator, and has the same arguments
@@ -351,7 +363,7 @@ class Blueprint(Scaffold):
             def after():
                 ...
         """
-        self.record_once(lambda state: state.app.after_websocket(func))
+        self.record_once(lambda state: state.app.after_serving(func))
         return func
 
     def teardown_app_request(self, func: TeardownCallable) -> TeardownCallable:
@@ -488,6 +500,8 @@ class Blueprint(Scaffold):
         Keyword arguments passed to this method will override the
         defaults set on the blueprint.
         """
+        if blueprint is self:
+            raise ValueError("Cannot register a blueprint on itself")
         self._blueprints.append((blueprint, options))
 
     def register(self, app: "Quart", options: dict) -> None:
@@ -500,17 +514,22 @@ class Blueprint(Scaffold):
             first_registration: Whether this is the first time this
                 blueprint has been registered on the application.
         """
-        first_registration = False
 
-        if self.name in app.blueprints and app.blueprints[self.name] is not self:
-            raise RuntimeError(
+        first_registration = True
+
+        for blueprint in app.blueprints.values():
+            if blueprint is self:
+                first_registration = False
+
+        name = f"{options.get('name_prefix', '')}.{options.get('name', self.name)}".lstrip(".")
+        if name in app.blueprints and app.blueprints[name] is not self:
+            raise ValueError(
                 f"Blueprint name '{self.name}' "
                 f"is already registered by {app.blueprints[self.name]}. "
                 "Blueprints must have unique names"
             )
-        else:
-            app.blueprints[self.name] = self
-            first_registration = True
+
+        app.blueprints[name] = self
 
         self._got_registered_once = True
         state = self.make_setup_state(app, options, first_registration)
@@ -526,11 +545,11 @@ class Blueprint(Scaffold):
 
             def extend(bp_dict: dict, parent_dict: dict) -> None:
                 for key, values in bp_dict.items():
-                    key = self.name if key is None else f"{self.name}.{key}"
+                    key = name if key is None else f"{name}.{key}"
                     parent_dict[key].extend(values)
 
             for key, value in self.error_handler_spec.items():
-                key = self.name if key is None else f"{self.name}.{key}"
+                key = name if key is None else f"{name}.{key}"
                 value = defaultdict(
                     dict,
                     {
@@ -568,19 +587,28 @@ class Blueprint(Scaffold):
             if cli_resolved_group is None:
                 app.cli.commands.update(self.cli.commands)
             elif cli_resolved_group is Ellipsis:
-                self.cli.name = self.name
+                self.cli.name = name
                 app.cli.add_command(self.cli)
             else:
                 self.cli.name = cli_resolved_group
                 app.cli.add_command(self.cli)
 
         for blueprint, bp_options in self._blueprints:
-            url_prefix = options.get("url_prefix", "")
-            if "url_prefix" in bp_options:
-                url_prefix = url_prefix.rstrip("/") + "/" + bp_options["url_prefix"].lstrip("/")
+            bp_options = bp_options.copy()
+            bp_url_prefix = bp_options.get("url_prefix")
+            if bp_url_prefix is None:
+                bp_url_prefix = blueprint.url_prefix
 
-            bp_options["url_prefix"] = url_prefix
-            bp_options["name_prefix"] = options.get("name_prefix", "") + self.name + "."
+            if state.url_prefix is not None and bp_url_prefix is not None:
+                bp_options["url_prefix"] = (
+                    state.url_prefix.rstrip("/") + "/" + bp_url_prefix.lstrip("/")
+                )
+            elif bp_url_prefix is not None:
+                bp_options["url_prefix"] = bp_url_prefix
+            elif state.url_prefix is not None:
+                bp_options["url_prefix"] = state.url_prefix
+
+            bp_options["name_prefix"] = name
             blueprint.register(app, bp_options)
 
     def make_setup_state(
@@ -622,6 +650,7 @@ class BlueprintSetupState:
         self.subdomain = options.get("subdomain") or blueprint.subdomain
         self.url_defaults = dict(self.blueprint.url_values_defaults)
         self.url_defaults.update(options.get("url_defaults", {}) or {})
+        self.name = self.options.get("name", blueprint.name)
         self.name_prefix = self.options.get("name_prefix", "")
 
     def add_url_rule(
@@ -637,12 +666,13 @@ class BlueprintSetupState:
         provide_automatic_options: Optional[bool] = None,
         is_websocket: bool = False,
         strict_slashes: Optional[bool] = None,
+        merge_slashes: Optional[bool] = None,
     ) -> None:
         if self.url_prefix is not None:
             path = f"{self.url_prefix.rstrip('/')}/{path.lstrip('/')}"
         if subdomain is None:
             subdomain = self.subdomain
-        endpoint = f"{self.name_prefix}{self.blueprint.name}.{endpoint}"
+        endpoint = f"{self.name_prefix}.{self.name}.{endpoint}".lstrip(".")
         url_defaults = self.url_defaults
         if defaults is not None:
             url_defaults = {**url_defaults, **defaults}
@@ -657,6 +687,7 @@ class BlueprintSetupState:
             subdomain=subdomain,
             is_websocket=is_websocket,
             strict_slashes=strict_slashes,
+            merge_slashes=merge_slashes,
         )
 
     def register_template_filter(self, func: TemplateFilterCallable, name: Optional[str]) -> None:
