@@ -7,7 +7,7 @@ import os
 import sys
 from importlib import import_module
 from pathlib import Path
-from typing import Any, Callable, Iterable, List, Optional, TYPE_CHECKING
+from typing import Any, Callable, List, Optional, TYPE_CHECKING
 
 import click
 
@@ -54,8 +54,8 @@ class ScriptInfo:
                     module_name, app_name = self.app_import_path.split(":", 1)
                 except ValueError:
                     module_name, app_name = self.app_import_path, "app"
-                except AttributeError:
-                    raise NoAppException()
+                except AttributeError as error:
+                    raise NoAppException() from error
 
                 module_path = Path(module_name).resolve()
                 sys.path.insert(0, str(module_path.parent))
@@ -67,14 +67,14 @@ class ScriptInfo:
                     module = import_module(import_name)
                 except ModuleNotFoundError as error:
                     if error.name == import_name:
-                        raise NoAppException()
+                        raise NoAppException() from error
                     else:
                         raise
 
                 try:
                     self._app = eval(app_name, vars(module))
-                except NameError:
-                    raise NoAppException()
+                except NameError as error:
+                    raise NoAppException() from error
 
                 from .app import Quart
 
@@ -120,7 +120,15 @@ def with_appcontext(fn: Optional[Callable] = None) -> Callable:
     def decorator(__ctx: click.Context, *args: Any, **kwargs: Any) -> Any:
         async def _inner() -> Any:
             async with __ctx.ensure_object(ScriptInfo).load_app().app_context():
-                return __ctx.invoke(fn, *args, **kwargs)
+                try:
+                    return __ctx.invoke(fn, *args, **kwargs)
+                except RuntimeError as error:
+                    if error.args[0] == "Cannot run the event loop while another loop is running":
+                        print(  # noqa: T001, T002
+                            "The appcontext cannot be used with a command that runs an event loop. "
+                            "See quart#361 for more details"
+                        )
+                    raise
 
         return asyncio.run(_inner())
 
@@ -138,9 +146,9 @@ class AppGroup(click.Group):
     def command(self, *args: Any, **kwargs: Any) -> Callable:
         """This works exactly like the method of the same name on a regular
         :class:`click.Group` but it wraps callbacks in :func:`with_appcontext`
-        unless it's disabled by passing ``with_appcontext=False``.
+        if it's enabled by passing ``with_appcontext=True``.
         """
-        wrap_for_ctx = kwargs.pop("with_appcontext", True)
+        wrap_for_ctx = kwargs.pop("with_appcontext", False)
 
         def decorator(f: Callable) -> Callable:
             if wrap_for_ctx:
@@ -191,6 +199,7 @@ class QuartGroup(AppGroup):
         if add_default_commands:
             self.add_command(run_command)
             self.add_command(shell_command)
+            self.add_command(routes_command)
 
     def get_command(self, ctx: click.Context, name: str) -> click.Command:
         """Return the relevant command given the context and name.
@@ -203,18 +212,18 @@ class QuartGroup(AppGroup):
         info = ctx.ensure_object(ScriptInfo)
         command = None
         try:
-            command = info.load_app().cli.get_command(ctx, name)
+            command = info.load_app().cli.get_command(ctx, name)  # type: ignore
         except NoAppException:
             pass
         if command is None:
             command = super().get_command(ctx, name)
         return command
 
-    def list_commands(self, ctx: click.Context) -> Iterable[str]:
+    def list_commands(self, ctx: click.Context) -> List[str]:
         commands = set(click.Group.list_commands(self, ctx))
         info = ctx.ensure_object(ScriptInfo)
-        commands.update(info.load_app().cli.list_commands(ctx))
-        return commands
+        commands.update(info.load_app().cli.list_commands(ctx))  # type: ignore
+        return list(commands)
 
     def main(self, *args: Any, **kwargs: Any) -> Any:
         kwargs.setdefault("obj", ScriptInfo(create_app=self.create_app))
@@ -256,6 +265,50 @@ def shell_command(info: ScriptInfo) -> None:
     code.interact(banner=banner, local=context)
 
 
+@click.command("routes", short_help="Show this app's routes.")
+@click.option(
+    "--sort",
+    "-s",
+    type=click.Choice({"endpoint", "methods", "rule", "match"}),  # type: ignore
+    default="endpoint",
+    help="Order the routes by type, 'match' is the matching order when dispatching a request.",
+)
+@click.option("--all-methods", is_flag=True, help="Show HEAD and OPTIONS methods.")
+@pass_script_info
+def routes_command(info: ScriptInfo, sort: str, all_methods: bool) -> None:
+    app = info.load_app()
+    rules = list(app.url_map.iter_rules())
+    if len(rules) == 0:
+        click.echo("No routes were registered.")
+        return
+
+    ignored_methods = set() if all_methods else {"HEAD", "OPTIONS"}
+
+    if sort == "endpoint":
+        rules = sorted(rules, key=lambda rule: rule.endpoint)
+    elif sort == "rule":
+        rules = sorted(rules, key=lambda rule: rule.rule)
+    elif sort == "methods":
+        rules = sorted(rules, key=lambda rule: sorted(rule.methods))
+
+    headers = ("Endpoint", "Methods", "Websocket", "Rule")
+    rule_methods = [", ".join(sorted(rule.methods - ignored_methods)) for rule in rules]
+    widths = (
+        max(len(rule.endpoint) for rule in rules),
+        max(len(methods) for methods in rule_methods),
+        len("Websocket"),
+        max(len(rule.rule) for rule in rules),
+    )
+    widths = [max(len(header), width) for header, width in zip(headers, widths)]
+    row = "{{0:<{0}}} | {{1:<{1}}} | {{2:<{2}}} | {{3:<{3}}}".format(*widths)
+
+    click.echo(row.format(*headers).strip())
+    click.echo(row.format(*("-" * width for width in widths)))
+
+    for rule, methods in zip(rules, rule_methods):
+        click.echo(row.format(rule.endpoint, methods, str(rule.websocket), rule.rule).rstrip())
+
+
 cli = QuartGroup(
     help="""\
 Utility functions for Quart applications.
