@@ -39,8 +39,6 @@ class _BaseRequestWebsocketContext:
         self.session = session
         self.preserved = False
 
-        self.match_request()
-
     @property
     def g(self) -> AppContext:
         return _app_ctx_stack.top.g
@@ -84,6 +82,9 @@ class _BaseRequestWebsocketContext:
             if self.session is None:
                 self.session = await session_interface.make_null_session(self.app)
 
+        if self.url_adapter is not None:
+            self.match_request()
+
     async def pop(self, exc: BaseException) -> None:
         await _app_ctx_stack.top.pop(exc)
 
@@ -439,3 +440,18 @@ class _AppCtxGlobals:
         if top is not None:
             return f"<quart.g of {top.app.name}>"
         return object.__repr__(self)
+
+    def __getattr__(self, name: str) -> Any:
+        try:
+            return self.__dict__[name]
+        except KeyError:
+            raise AttributeError(name) from None
+
+    def __setattr__(self, name: str, value: Any) -> None:
+        self.__dict__[name] = value
+
+    def __delattr__(self, name: str) -> None:
+        try:
+            del self.__dict__[name]
+        except KeyError:
+            raise AttributeError(name) from None
