@@ -5,7 +5,7 @@ import os
 import pkgutil
 import sys
 from datetime import datetime, timedelta
-from functools import wraps
+from functools import lru_cache, wraps
 from io import BytesIO
 from pathlib import Path
 from typing import Any, Callable, Iterable, List, Optional, Tuple, Union
@@ -206,7 +206,7 @@ def url_for(
 
     if url_adapter is None:
         raise RuntimeError(
-            "Unable to create a url adapter, try setting the the SERVER_NAME config variable."
+            "Unable to create a url adapter, try setting the SERVER_NAME config variable."
         )
     if _scheme is not None and not _external:
         raise ValueError("External must be True for scheme usage")
@@ -417,3 +417,11 @@ async def send_file(
     if conditional:
         await response.make_conditional(request.range)
     return response
+
+
+@lru_cache(maxsize=None)
+def _split_blueprint_path(name: str) -> List[str]:
+    bps = [name]
+    while "." in bps[-1]:
+        bps.append(bps[-1].rpartition(".")[0])
+    return bps
