@@ -1,10 +1,11 @@
 from __future__ import annotations
 
 import json
+from dataclasses import asdict, is_dataclass
 from datetime import date
 from email.utils import formatdate
 from time import mktime
-from typing import Any, Optional, Type, TYPE_CHECKING
+from typing import Any, Dict, IO, Optional, Type, TYPE_CHECKING
 from uuid import UUID
 
 from jinja2 import Markup
@@ -16,10 +17,10 @@ if TYPE_CHECKING:
     from ..wrappers import Response  # noqa: F401
 
 
-def dumps(object_: Any, app: Optional["Quart"] = None, **kwargs: Any) -> str:
+def _dump_arg_defaults(kwargs: Dict[str, Any], app: Optional["Quart"] = None) -> Dict[str, Any]:
     json_encoder: Type[json.JSONEncoder] = JSONEncoder
     if app is None and _app_ctx_stack.top is not None:  # has_app_context requires a circular import
-        app = current_app._get_current_object()
+        app = current_app._get_current_object()  # type: ignore
 
     if app is not None:
         json_encoder = app.json_encoder
@@ -31,14 +32,23 @@ def dumps(object_: Any, app: Optional["Quart"] = None, **kwargs: Any) -> str:
         kwargs.setdefault("sort_keys", app.config["JSON_SORT_KEYS"])
     kwargs.setdefault("sort_keys", True)
     kwargs.setdefault("cls", json_encoder)
+    return kwargs
 
+
+def dumps(object_: Any, app: Optional["Quart"] = None, **kwargs: Any) -> str:
+    kwargs = _dump_arg_defaults(kwargs, app)
     return json.dumps(object_, **kwargs)
 
 
-def loads(object_: str, app: Optional["Quart"] = None, **kwargs: Any) -> Any:
+def dump(object_: Any, fp: IO[str], app: Optional["Quart"] = None, **kwargs: Any) -> None:
+    kwargs = _dump_arg_defaults(kwargs, app)
+    json.dump(object_, fp, **kwargs)
+
+
+def _load_arg_defaults(kwargs: Dict[str, Any], app: Optional["Quart"] = None) -> Dict[str, Any]:
     json_decoder: Type[json.JSONDecoder] = JSONDecoder
     if app is None and _app_ctx_stack.top is not None:  # has_app_context requires a circular import
-        app = current_app._get_current_object()
+        app = current_app._get_current_object()  # type: ignore
 
     if app is not None:
         json_decoder = app.json_decoder
@@ -47,10 +57,19 @@ def loads(object_: str, app: Optional["Quart"] = None, **kwargs: Any) -> Any:
             if blueprint is not None and blueprint.json_decoder is not None:
                 json_decoder = blueprint.json_decoder
     kwargs.setdefault("cls", json_decoder)
+    return kwargs
 
+
+def loads(object_: str, app: Optional["Quart"] = None, **kwargs: Any) -> Any:
+    kwargs = _load_arg_defaults(kwargs, app)
     return json.loads(object_, **kwargs)
 
 
+def load(fp: IO[str], app: Optional["Quart"] = None, **kwargs: Any) -> Any:
+    kwargs = _load_arg_defaults(kwargs, app)
+    return json.load(fp, **kwargs)
+
+
 def htmlsafe_dumps(object_: Any, **kwargs: Any) -> str:
     return (
         dumps(object_, **kwargs)
@@ -89,6 +108,8 @@ class JSONEncoder(json.JSONEncoder):
             return formatdate(timeval=mktime((object_.timetuple())), localtime=False, usegmt=True)
         if isinstance(object_, UUID):
             return str(object_)
+        if is_dataclass(object_):
+            return asdict(object_)
         if hasattr(object_, "__html__"):
             return str(object_.__html__())
         return super().default(object_)
