@@ -1,16 +1,26 @@
 from __future__ import annotations
 
 import asyncio
-import functools
 import inspect
 import sys
-import warnings
 from contextvars import copy_context
 from functools import partial, wraps
-from inspect import isgenerator
 from os import PathLike
 from pathlib import Path
-from typing import Any, AsyncGenerator, Callable, Coroutine, Generator, List, TYPE_CHECKING, Union
+from typing import (
+    Any,
+    AsyncGenerator,
+    Callable,
+    Coroutine,
+    Generator,
+    Iterable,
+    List,
+    Tuple,
+    TYPE_CHECKING,
+    Union,
+)
+
+from werkzeug.datastructures import Headers
 
 from .globals import current_app
 from .typing import FilePath
@@ -30,20 +40,6 @@ You should be redirected to <a href="{location}">{location}</a>, if not please c
     return current_app.response_class(body, status=code, headers={"Location": location})
 
 
-def ensure_coroutine(func: Callable) -> Callable:
-    warnings.warn(
-        "Please switch to using a coroutine function. "
-        "Synchronous functions will not be supported in 0.13 onwards.",
-        DeprecationWarning,
-    )
-    if is_coroutine_function(func):
-        return func
-    else:
-        async_func = asyncio.coroutine(func)
-        async_func._quart_async_wrapper = True  # type: ignore
-        return async_func
-
-
 def file_path_to_path(*paths: FilePath) -> Path:
     # Flask supports bytes paths
     safe_paths: List[Union[str, PathLike]] = []
@@ -55,7 +51,7 @@ def file_path_to_path(*paths: FilePath) -> Path:
     return Path(*safe_paths)
 
 
-def run_sync(func: Callable[..., Any]) -> Callable[..., Coroutine[Any, None, None]]:
+def run_sync(func: Callable[..., Any]) -> Callable[..., Coroutine[None, None, Any]]:
     """Ensure that the sync function is run within the event loop.
 
     If the *func* is not a coroutine it will be wrapped such that
@@ -70,7 +66,7 @@ def run_sync(func: Callable[..., Any]) -> Callable[..., Coroutine[Any, None, Non
         result = await loop.run_in_executor(
             None, copy_context().run, partial(func, *args, **kwargs)
         )
-        if isgenerator(result):
+        if inspect.isgenerator(result):
             return run_sync_iterable(result)  # type: ignore
         else:
             return result
@@ -124,9 +120,17 @@ def is_coroutine_function(func: Any) -> bool:
 
         while inspect.ismethod(func):
             func = func.__func__
-        while isinstance(func, functools.partial):
+        while isinstance(func, partial):
             func = func.func
         if not inspect.isfunction(func):
             return False
         result = bool(func.__code__.co_flags & inspect.CO_COROUTINE)
         return result or getattr(func, "_is_coroutine", None) is asyncio.coroutines._is_coroutine
+
+
+def encode_headers(headers: Headers) -> List[Tuple[bytes, bytes]]:
+    return [(key.lower().encode(), value.encode()) for key, value in headers.items()]
+
+
+def decode_headers(headers: Iterable[Tuple[bytes, bytes]]) -> Headers:
+    return Headers([(key.decode(), value.decode()) for key, value in headers])
